@@ -178,6 +178,8 @@ public class ZooKeeperServerMain {
             containerManager.start();
             ZKAuditProvider.addZKStartStopAuditLog();
 
+            serverStarted();
+
             // Watch status of ZooKeeper server. It will do a graceful shutdown
             // if the server is not running or hits an internal error.
             shutdownLatch.await();
@@ -247,21 +249,35 @@ public class ZooKeeperServerMain {
      */
     public void close() {
         ServerCnxnFactory primaryCnxnFactory = this.cnxnFactory;
-        if (primaryCnxnFactory == null) {
-            // in case of pure TLS we can hook into secureCnxnFactory
-            primaryCnxnFactory = secureCnxnFactory;
-        }
-        if (primaryCnxnFactory == null || primaryCnxnFactory.getZooKeeperServer() == null) {
-            return;
-        }
-        ZooKeeperServerShutdownHandler zkShutdownHandler = primaryCnxnFactory.getZooKeeperServer().getZkShutdownHandler();
-        zkShutdownHandler.handle(ZooKeeperServer.State.SHUTDOWN);
+        ServerCnxnFactory secondaryCnxnFactory = this.secureCnxnFactory;
         try {
-            // ServerCnxnFactory will call the shutdown
-            primaryCnxnFactory.join();
-        } catch (InterruptedException ex) {
-            Thread.currentThread().interrupt();
+            if (primaryCnxnFactory == null) {
+                // in case of pure TLS we can hook into secureCnxnFactory
+                primaryCnxnFactory = secondaryCnxnFactory;
+            }
+            if (primaryCnxnFactory == null || primaryCnxnFactory.getZooKeeperServer() == null) {
+                LOG.info("Connection factory did not start");
+                return;
+            }
+            ZooKeeperServerShutdownHandler zkShutdownHandler = primaryCnxnFactory.getZooKeeperServer().getZkShutdownHandler();
+            zkShutdownHandler.handle(ZooKeeperServer.State.SHUTDOWN);
+            try {
+                // ServerCnxnFactory will call the shutdown
+                primaryCnxnFactory.join();
+            } catch (InterruptedException ex) {
+                Thread.currentThread().interrupt();
+            }
+        } finally {
+            // ensure that we are closing the sockets
+            if (primaryCnxnFactory != null) {
+                primaryCnxnFactory.shutdown();
+            }
+            if (secondaryCnxnFactory != null) {
+                secondaryCnxnFactory.shutdown();
+            }
         }
     }
 
+    protected void serverStarted() {
+    }
 }
