@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,28 +18,40 @@
 
 package org.apache.zookeeper.server.admin;
 
-import java.util.ArrayList;
+import com.fasterxml.jackson.annotation.JsonProperty;
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
+import java.net.InetSocketAddress;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Set;
-
+import java.util.SortedMap;
+import java.util.TreeMap;
+import java.util.stream.Collectors;
 import org.apache.zookeeper.Environment;
 import org.apache.zookeeper.Environment.Entry;
 import org.apache.zookeeper.Version;
 import org.apache.zookeeper.server.DataTree;
 import org.apache.zookeeper.server.ServerCnxnFactory;
-import org.apache.zookeeper.server.ServerStats;
-import org.apache.zookeeper.server.ZKDatabase;
+import org.apache.zookeeper.server.ServerMetrics;
 import org.apache.zookeeper.server.ZooKeeperServer;
 import org.apache.zookeeper.server.ZooTrace;
 import org.apache.zookeeper.server.persistence.SnapshotInfo;
-import org.apache.zookeeper.server.quorum.*;
+import org.apache.zookeeper.server.quorum.Follower;
+import org.apache.zookeeper.server.quorum.FollowerZooKeeperServer;
+import org.apache.zookeeper.server.quorum.Leader;
+import org.apache.zookeeper.server.quorum.LeaderZooKeeperServer;
+import org.apache.zookeeper.server.quorum.MultipleAddresses;
+import org.apache.zookeeper.server.quorum.QuorumPeer;
+import org.apache.zookeeper.server.quorum.QuorumPeer.LearnerType;
+import org.apache.zookeeper.server.quorum.QuorumZooKeeperServer;
+import org.apache.zookeeper.server.quorum.ReadOnlyZooKeeperServer;
 import org.apache.zookeeper.server.quorum.flexible.QuorumVerifier;
-import org.apache.zookeeper.server.util.OSMXBean;
+import org.apache.zookeeper.server.util.ZxidUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -51,6 +63,7 @@ import org.slf4j.LoggerFactory;
  * @see JettyAdminServer
  */
 public class Commands {
+
     static final Logger LOG = LoggerFactory.getLogger(Commands.class);
 
     /** Maps command names to Command instances */
@@ -65,7 +78,7 @@ public class Commands {
         for (String name : command.getNames()) {
             Command prev = commands.put(name, command);
             if (prev != null) {
-                LOG.warn("Re-registering command %s (primary name = %s)", name, command.getPrimaryName());
+                LOG.warn("Re-registering command {} (primary name = {})", name, command.getPrimaryName());
             }
         }
         primaryNames.add(command.getPrimaryName());
@@ -85,14 +98,18 @@ public class Commands {
      *    - "command" key containing the command's primary name
      *    - "error" key containing a String error message or null if no error
      */
-    public static CommandResponse runCommand(String cmdName, ZooKeeperServer zkServer, Map<String, String> kwargs) {
-        if (!commands.containsKey(cmdName)) {
+    public static CommandResponse runCommand(
+        String cmdName,
+        ZooKeeperServer zkServer,
+        Map<String, String> kwargs) {
+        Command command = getCommand(cmdName);
+        if (command == null) {
             return new CommandResponse(cmdName, "Unknown command: " + cmdName);
         }
-        if (zkServer == null || !zkServer.isRunning()) {
+        if (command.isServerRequired() && (zkServer == null || !zkServer.isRunning())) {
             return new CommandResponse(cmdName, "This ZooKeeper instance is not currently serving requests");
         }
-        return commands.get(cmdName).run(zkServer, kwargs);
+        return command.run(zkServer, kwargs);
     }
 
     /**
@@ -114,18 +131,25 @@ public class Commands {
         registerCommand(new CnxnStatResetCommand());
         registerCommand(new ConfCommand());
         registerCommand(new ConsCommand());
+        registerCommand(new DigestCommand());
         registerCommand(new DirsCommand());
         registerCommand(new DumpCommand());
         registerCommand(new EnvCommand());
         registerCommand(new GetTraceMaskCommand());
+        registerCommand(new InitialConfigurationCommand());
         registerCommand(new IsroCommand());
         registerCommand(new LastSnapshotCommand());
+        registerCommand(new LeaderCommand());
         registerCommand(new MonitorCommand());
+        registerCommand(new ObserverCnxnStatResetCommand());
         registerCommand(new RuokCommand());
         registerCommand(new SetTraceMaskCommand());
         registerCommand(new SrvrCommand());
         registerCommand(new StatCommand());
         registerCommand(new StatResetCommand());
+        registerCommand(new SyncedObserverConsCommand());
+        registerCommand(new SystemPropertiesCommand());
+        registerCommand(new VotingViewCommand());
         registerCommand(new WatchCommand());
         registerCommand(new WatchesByPathCommand());
         registerCommand(new WatchSummaryCommand());
@@ -136,6 +160,7 @@ public class Commands {
      * Reset all connection statistics.
      */
     public static class CnxnStatResetCommand extends CommandBase {
+
         public CnxnStatResetCommand() {
             super(Arrays.asList("connection_stat_reset", "crst"));
         }
@@ -147,6 +172,7 @@ public class Commands {
             return response;
 
         }
+
     }
 
     /**
@@ -154,6 +180,7 @@ public class Commands {
      * @see ZooKeeperServer#getConf()
      */
     public static class ConfCommand extends CommandBase {
+
         public ConfCommand() {
             super(Arrays.asList("configuration", "conf", "config"));
         }
@@ -164,6 +191,7 @@ public class Commands {
             response.putAll(zkServer.getConf().toMap());
             return response;
         }
+
     }
 
     /**
@@ -172,6 +200,7 @@ public class Commands {
      * @see org.apache.zookeeper.server.ServerCnxn#getConnectionInfo(boolean)
      */
     public static class ConsCommand extends CommandBase {
+
         public ConsCommand() {
             super(Arrays.asList("connections", "cons"));
         }
@@ -193,12 +222,14 @@ public class Commands {
             }
             return response;
         }
+
     }
 
     /**
      * Information on ZK datadir and snapdir size in bytes
      */
     public static class DirsCommand extends CommandBase {
+
         public DirsCommand() {
             super(Arrays.asList("dirs"));
         }
@@ -210,18 +241,20 @@ public class Commands {
             response.put("logdir_size", zkServer.getLogDirSize());
             return response;
         }
+
     }
 
     /**
      * Information on session expirations and ephemerals. Returned map contains:
-     *   - "expiry_time_to_session_ids": Map<Long, Set<Long>>
-     *                                   time -> sessions IDs of sessions that expire at time
-     *   - "sesssion_id_to_ephemeral_paths": Map<Long, Set<String>>
-     *                                       session ID -> ephemeral paths created by that session
+     *   - "expiry_time_to_session_ids": Map&lt;Long, Set&lt;Long&gt;&gt;
+     *                                   time -&gt; sessions IDs of sessions that expire at time
+     *   - "session_id_to_ephemeral_paths": Map&lt;Long, Set&lt;String&gt;&gt;
+     *                                       session ID -&gt; ephemeral paths created by that session
      * @see ZooKeeperServer#getSessionExpiryMap()
      * @see ZooKeeperServer#getEphemerals()
      */
     public static class DumpCommand extends CommandBase {
+
         public DumpCommand() {
             super(Arrays.asList("dump"));
         }
@@ -233,14 +266,16 @@ public class Commands {
             response.put("session_id_to_ephemeral_paths", zkServer.getEphemerals());
             return response;
         }
+
     }
 
     /**
      * All defined environment variables.
      */
     public static class EnvCommand extends CommandBase {
+
         public EnvCommand() {
-            super(Arrays.asList("environment", "env", "envi"));
+            super(Arrays.asList("environment", "env", "envi"), false);
         }
 
         @Override
@@ -251,6 +286,25 @@ public class Commands {
             }
             return response;
         }
+
+    }
+
+    /**
+     * Digest histories for every specific number of txns.
+     */
+    public static class DigestCommand extends CommandBase {
+
+        public DigestCommand() {
+            super(Arrays.asList("hash"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            response.put("digests", zkServer.getZKDatabase().getDataTree().getDigestLog());
+            return response;
+        }
+
     }
 
     /**
@@ -258,8 +312,9 @@ public class Commands {
      *   - "tracemask": Long
      */
     public static class GetTraceMaskCommand extends CommandBase {
+
         public GetTraceMaskCommand() {
-            super(Arrays.asList("get_trace_mask", "gtmk"));
+            super(Arrays.asList("get_trace_mask", "gtmk"), false);
         }
 
         @Override
@@ -268,6 +323,22 @@ public class Commands {
             response.put("tracemask", ZooTrace.getTextTraceLevel());
             return response;
         }
+
+    }
+
+    public static class InitialConfigurationCommand extends CommandBase {
+
+        public InitialConfigurationCommand() {
+            super(Arrays.asList("initial_configuration", "icfg"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            response.put("initial_configuration", zkServer.getInitialConfig());
+            return response;
+        }
+
     }
 
     /**
@@ -275,6 +346,7 @@ public class Commands {
      *   - "is_read_only": Boolean
      */
     public static class IsroCommand extends CommandBase {
+
         public IsroCommand() {
             super(Arrays.asList("is_read_only", "isro"));
         }
@@ -285,6 +357,7 @@ public class Commands {
             response.put("read_only", zkServer instanceof ReadOnlyZooKeeperServer);
             return response;
         }
+
     }
 
     /**
@@ -297,6 +370,7 @@ public class Commands {
      *   - "timestamp": Long
      */
     public static class LastSnapshotCommand extends CommandBase {
+
         public LastSnapshotCommand() {
             super(Arrays.asList("last_snapshot", "lsnp"));
         }
@@ -309,6 +383,33 @@ public class Commands {
             response.put("timestamp", info == null ? -1L : info.timestamp);
             return response;
         }
+
+    }
+
+    /**
+     * Returns the leader status of this instance and the leader host string.
+     */
+    public static class LeaderCommand extends CommandBase {
+
+        public LeaderCommand() {
+            super(Arrays.asList("leader", "lead"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            if (zkServer instanceof QuorumZooKeeperServer) {
+                response.put("is_leader", zkServer instanceof LeaderZooKeeperServer);
+                QuorumPeer peer = ((QuorumZooKeeperServer) zkServer).self;
+                response.put("leader_id", peer.getLeaderId());
+                String leaderAddress = peer.getLeaderAddress();
+                response.put("leader_ip", leaderAddress != null ? leaderAddress : "");
+            } else {
+                response.put("error", "server is not initialized");
+            }
+            return response;
+        }
+
     }
 
     /**
@@ -329,71 +430,61 @@ public class Commands {
      *   - "ephemerals_count": Integer
      *   - "approximate_data_size": Long
      *   - "open_file_descriptor_count": Long (unix only)
-     *   - "max_file_descritpor_count": Long (unix only)
+     *   - "max_file_descriptor_count": Long (unix only)
      *   - "fsync_threshold_exceed_count": Long
+     *   - "non_mtls_conn_count": Long
+     *   - "non_mtls_remote_conn_count": Long
+     *   - "non_mtls_local_conn_count": Long
      *   - "followers": Integer (leader only)
      *   - "synced_followers": Integer (leader only)
      *   - "pending_syncs": Integer (leader only)
      */
     public static class MonitorCommand extends CommandBase {
+
         public MonitorCommand() {
-            super(Arrays.asList("monitor", "mntr"));
+            super(Arrays.asList("monitor", "mntr"), false);
         }
 
         @Override
         public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
-            ZKDatabase zkdb = zkServer.getZKDatabase();
-            ServerStats stats = zkServer.serverStats();
-
             CommandResponse response = initializeResponse();
-
-            response.put("version", Version.getFullVersion());
-
-            response.put("avg_latency", stats.getAvgLatency());
-            response.put("max_latency", stats.getMaxLatency());
-            response.put("min_latency", stats.getMinLatency());
-
-            response.put("packets_received", stats.getPacketsReceived());
-            response.put("packets_sent", stats.getPacketsSent());
-            response.put("num_alive_connections", stats.getNumAliveClientConnections());
-
-            response.put("outstanding_requests", stats.getOutstandingRequests());
-
-            response.put("server_state", stats.getServerState());
-            response.put("znode_count", zkdb.getNodeCount());
-
-            response.put("watch_count", zkdb.getDataTree().getWatchCount());
-            response.put("ephemerals_count", zkdb.getDataTree().getEphemeralsCount());
-            response.put("approximate_data_size", zkdb.getDataTree().approximateDataSize());
-
-            OSMXBean osMbean = new OSMXBean();
-            response.put("open_file_descriptor_count", osMbean.getOpenFileDescriptorCount());
-            response.put("max_file_descriptor_count", osMbean.getMaxFileDescriptorCount());
-
-            response.put("last_client_response_size", stats.getClientResponseStats().getLastBufferSize());
-            response.put("max_client_response_size", stats.getClientResponseStats().getMaxBufferSize());
-            response.put("min_client_response_size", stats.getClientResponseStats().getMinBufferSize());
-
-            if (zkServer instanceof LeaderZooKeeperServer) {
-                Leader leader = ((LeaderZooKeeperServer) zkServer).getLeader();
-
-                response.put("followers", leader.getLearners().size());
-                response.put("synced_followers", leader.getForwardingFollowers().size());
-                response.put("pending_syncs", leader.getNumPendingSyncs());
-
-                response.put("last_proposal_size", leader.getProposalStats().getLastBufferSize());
-                response.put("max_proposal_size", leader.getProposalStats().getMaxBufferSize());
-                response.put("min_proposal_size", leader.getProposalStats().getMinBufferSize());
-            }
-
+            zkServer.dumpMonitorValues(response::put);
+            ServerMetrics.getMetrics().getMetricsProvider().dump(response::put);
             return response;
 
-        }}
+        }
+
+    }
+
+    /**
+     * Reset all observer connection statistics.
+     */
+    public static class ObserverCnxnStatResetCommand extends CommandBase {
+
+        public ObserverCnxnStatResetCommand() {
+            super(Arrays.asList("observer_connection_stat_reset", "orst"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            if (zkServer instanceof LeaderZooKeeperServer) {
+                Leader leader = ((LeaderZooKeeperServer) zkServer).getLeader();
+                leader.resetObserverConnectionStats();
+            } else if (zkServer instanceof FollowerZooKeeperServer) {
+                Follower follower = ((FollowerZooKeeperServer) zkServer).getFollower();
+                follower.resetObserverConnectionStats();
+            }
+            return response;
+        }
+
+    }
 
     /**
      * No-op command, check if the server is running
      */
     public static class RuokCommand extends CommandBase {
+
         public RuokCommand() {
             super(Arrays.asList("ruok"));
         }
@@ -402,6 +493,7 @@ public class Commands {
         public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
             return initializeResponse();
         }
+
     }
 
     /**
@@ -411,8 +503,9 @@ public class Commands {
      *   - "tracemask": Long
      */
     public static class SetTraceMaskCommand extends CommandBase {
+
         public SetTraceMaskCommand() {
-            super(Arrays.asList("set_trace_mask", "stmk"));
+            super(Arrays.asList("set_trace_mask", "stmk"), false);
         }
 
         @Override
@@ -426,8 +519,7 @@ public class Commands {
             try {
                 traceMask = Long.parseLong(kwargs.get("traceMask"));
             } catch (NumberFormatException e) {
-                response.put("error", "setTraceMask requires long traceMask argument, got "
-                                      + kwargs.get("traceMask"));
+                response.put("error", "setTraceMask requires long traceMask argument, got " + kwargs.get("traceMask"));
                 return response;
             }
 
@@ -435,6 +527,7 @@ public class Commands {
             response.put("tracemask", traceMask);
             return response;
         }
+
     }
 
     /**
@@ -447,6 +540,7 @@ public class Commands {
      *   - "node_count": Integer
      */
     public static class SrvrCommand extends CommandBase {
+
         public SrvrCommand() {
             super(Arrays.asList("server_stats", "srvr"));
         }
@@ -465,18 +559,20 @@ public class Commands {
             response.put("server_stats", zkServer.serverStats());
             response.put("client_response", zkServer.serverStats().getClientResponseStats());
             if (zkServer instanceof LeaderZooKeeperServer) {
-                Leader leader = ((LeaderZooKeeperServer)zkServer).getLeader();
+                Leader leader = ((LeaderZooKeeperServer) zkServer).getLeader();
                 response.put("proposal_stats", leader.getProposalStats());
             }
             response.put("node_count", zkServer.getZKDatabase().getNodeCount());
             return response;
         }
+
     }
 
     /**
      * Same as SrvrCommand but has extra "connections" entry.
      */
     public static class StatCommand extends SrvrCommand {
+
         public StatCommand() {
             super(Arrays.asList("stats", "stat"));
         }
@@ -502,12 +598,14 @@ public class Commands {
             response.put("secure_connections", secureConnections);
             return response;
         }
+
     }
 
     /**
      * Resets server statistics.
      */
     public static class StatResetCommand extends CommandBase {
+
         public StatResetCommand() {
             super(Arrays.asList("stat_reset", "srst"));
         }
@@ -518,14 +616,144 @@ public class Commands {
             zkServer.serverStats().reset();
             return response;
         }
+
+    }
+
+    /**
+     * Information on observer connections to server. Returned Map contains:
+     *   - "synced_observers": Integer (leader/follower only)
+     *   - "observers": list of observer learner handler info objects (leader/follower only)
+     * @see org.apache.zookeeper.server.quorum.LearnerHandler#getLearnerHandlerInfo()
+     */
+    public static class SyncedObserverConsCommand extends CommandBase {
+
+        public SyncedObserverConsCommand() {
+            super(Arrays.asList("observers", "obsr"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+
+            CommandResponse response = initializeResponse();
+
+            if (zkServer instanceof LeaderZooKeeperServer) {
+                Leader leader = ((LeaderZooKeeperServer) zkServer).getLeader();
+
+                response.put("synced_observers", leader.getObservingLearners().size());
+                response.put("observers", leader.getObservingLearnersInfo());
+                return response;
+            } else if (zkServer instanceof FollowerZooKeeperServer) {
+                Follower follower = ((FollowerZooKeeperServer) zkServer).getFollower();
+                Integer syncedObservers = follower.getSyncedObserverSize();
+                if (syncedObservers != null) {
+                    response.put("synced_observers", syncedObservers);
+                    response.put("observers", follower.getSyncedObserversInfo());
+                    return response;
+                }
+            }
+
+            response.put("synced_observers", 0);
+            response.put("observers", Collections.emptySet());
+            return response;
+        }
+
+    }
+
+    /**
+     * All defined system properties.
+     */
+    public static class SystemPropertiesCommand extends CommandBase {
+
+        public SystemPropertiesCommand() {
+            super(Arrays.asList("system_properties", "sysp"), false);
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            Properties systemProperties = System.getProperties();
+            SortedMap<String, String> sortedSystemProperties = new TreeMap<>();
+            systemProperties.forEach((k, v) -> sortedSystemProperties.put(k.toString(), v.toString()));
+            response.putAll(sortedSystemProperties);
+            return response;
+        }
+
+    }
+
+    /**
+     * Returns the current ensemble configuration information.
+     * It provides list of current voting members in the ensemble.
+     */
+    public static class VotingViewCommand extends CommandBase {
+
+        public VotingViewCommand() {
+            super(Arrays.asList("voting_view"));
+        }
+
+        @Override
+        public CommandResponse run(ZooKeeperServer zkServer, Map<String, String> kwargs) {
+            CommandResponse response = initializeResponse();
+            if (zkServer instanceof QuorumZooKeeperServer) {
+                QuorumPeer peer = ((QuorumZooKeeperServer) zkServer).self;
+                Map<Long, QuorumServerView> votingView = peer.getVotingView().entrySet().stream()
+                        .collect(Collectors.toMap(Map.Entry::getKey, e -> new QuorumServerView(e.getValue())));
+                response.put("current_config", votingView);
+            } else {
+                response.put("current_config", Collections.emptyMap());
+            }
+            return response;
+        }
+
+        @SuppressFBWarnings(value = "URF_UNREAD_FIELD", justification = "class is used only for JSON serialization")
+        private static class QuorumServerView {
+
+            @JsonProperty
+            private List<String> serverAddresses;
+
+            @JsonProperty
+            private List<String> electionAddresses;
+
+            @JsonProperty
+            private String clientAddress;
+
+            @JsonProperty
+            private String learnerType;
+
+            public QuorumServerView(QuorumPeer.QuorumServer quorumServer) {
+                this.serverAddresses = getMultiAddressString(quorumServer.addr);
+                this.electionAddresses = getMultiAddressString(quorumServer.electionAddr);
+                this.learnerType = quorumServer.type.equals(LearnerType.PARTICIPANT) ? "participant" : "observer";
+                this.clientAddress = getAddressString(quorumServer.clientAddr);
+            }
+
+            private static List<String> getMultiAddressString(MultipleAddresses multipleAddresses) {
+                if (multipleAddresses == null) {
+                    return Collections.emptyList();
+                }
+
+                return multipleAddresses.getAllAddresses().stream()
+                        .map(QuorumServerView::getAddressString)
+                        .collect(Collectors.toList());
+            }
+
+            private static String getAddressString(InetSocketAddress address) {
+                if (address == null) {
+                    return "";
+                }
+                return String.format("%s:%d", QuorumPeer.QuorumServer.delimitedHostString(address), address.getPort());
+            }
+       }
+
     }
 
     /**
      * Watch information aggregated by session. Returned Map contains:
-     *   - "session_id_to_watched_paths": Map<Long, Set<String>> session ID -> watched paths
+     *   - "session_id_to_watched_paths": Map&lt;Long, Set&lt;String&gt;&gt; session ID -&gt; watched paths
+     * @see DataTree#getWatches()
      * @see DataTree#getWatches()
      */
     public static class WatchCommand extends CommandBase {
+
         public WatchCommand() {
             super(Arrays.asList("watches", "wchc"));
         }
@@ -537,14 +765,16 @@ public class Commands {
             response.put("session_id_to_watched_paths", dt.getWatches().toMap());
             return response;
         }
+
     }
 
     /**
      * Watch information aggregated by path. Returned Map contains:
-     *   - "path_to_session_ids": Map<String, Set<Long>> path -> session IDs of sessions watching path
+     *   - "path_to_session_ids": Map&lt;String, Set&lt;Long&gt;&gt; path -&gt; session IDs of sessions watching path
      * @see DataTree#getWatchesByPath()
      */
     public static class WatchesByPathCommand extends CommandBase {
+
         public WatchesByPathCommand() {
             super(Arrays.asList("watches_by_path", "wchp"));
         }
@@ -556,6 +786,7 @@ public class Commands {
             response.put("path_to_session_ids", dt.getWatchesByPath().toMap());
             return response;
         }
+
     }
 
     /**
@@ -563,6 +794,7 @@ public class Commands {
      * @see DataTree#getWatchesSummary()
      */
     public static class WatchSummaryCommand extends CommandBase {
+
         public WatchSummaryCommand() {
             super(Arrays.asList("watch_summary", "wchs"));
         }
@@ -574,16 +806,17 @@ public class Commands {
             response.putAll(dt.getWatchesSummary().toMap());
             return response;
         }
-    }
 
+    }
 
     /**
      * Returns the current phase of Zab protocol that peer is running.
      * It can be in one of these phases: ELECTION, DISCOVERY, SYNCHRONIZATION, BROADCAST
      */
     public static class ZabStateCommand extends CommandBase {
+
         public ZabStateCommand() {
-            super(Arrays.asList("zabstate"));
+            super(Arrays.asList("zabstate"), false);
         }
 
         @Override
@@ -594,21 +827,33 @@ public class Commands {
                 QuorumPeer.ZabState zabState = peer.getZabState();
                 QuorumVerifier qv = peer.getQuorumVerifier();
 
-                QuorumPeer.QuorumServer voter = qv.getVotingMembers().get(peer.getId());
+                QuorumPeer.QuorumServer voter = qv.getVotingMembers().get(peer.getMyId());
                 boolean voting = (
                         voter != null
                                 && voter.addr.equals(peer.getQuorumAddress())
                                 && voter.electionAddr.equals(peer.getElectionAddress())
                 );
+                response.put("myid", zkServer.getConf().getServerId());
+                response.put("is_leader", zkServer instanceof LeaderZooKeeperServer);
+                response.put("quorum_address", peer.getQuorumAddress());
+                response.put("election_address", peer.getElectionAddress());
+                response.put("client_address", peer.getClientAddress());
                 response.put("voting", voting);
+                long lastProcessedZxid = zkServer.getZKDatabase().getDataTreeLastProcessedZxid();
+                response.put("last_zxid", "0x" + ZxidUtils.zxidToString(lastProcessedZxid));
+                response.put("zab_epoch", ZxidUtils.getEpochFromZxid(lastProcessedZxid));
+                response.put("zab_counter", ZxidUtils.getCounterFromZxid(lastProcessedZxid));
                 response.put("zabstate", zabState.name().toLowerCase());
             } else {
                 response.put("voting", false);
                 response.put("zabstate", "");
             }
-            return response ;
+            return response;
         }
+
+    }
+
+    private Commands() {
     }
 
-    private Commands() {}
 }
