@@ -20,11 +20,14 @@ package org.apache.zookeeper;
 
 import static org.junit.Assume.assumeTrue;
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertIterableEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.UUID;
 import org.apache.zookeeper.data.Stat;
@@ -71,7 +74,7 @@ public class ZKUtilTest extends ClientBase {
         String absolutePath = file.getAbsolutePath();
         String error = ZKUtil.validateFileInput(absolutePath);
         assertNotNull(error);
-        String expectedMessage = "'" + absolutePath + "' is a direcory. it must be a file.";
+        String expectedMessage = "'" + absolutePath + "' is a directory. it must be a file.";
         assertEquals(expectedMessage, error);
     }
 
@@ -89,6 +92,35 @@ public class ZKUtilTest extends ClientBase {
         assertEquals(expectedMessage, error);
     }
 
+    @Test
+    public void testListRootPathSuccess() throws IOException, InterruptedException, KeeperException {
+        TestableZooKeeper zk = createClient();
+        zk.setData("/", "some".getBytes(), -1);
+        zk.create("/a", "some".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        zk.create("/a/b", "some".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+
+        List<String> list = ZKUtil.listSubTreeBFS(zk, "/");
+        list.remove(Quotas.procZookeeper);
+        list.remove(Quotas.quotaZookeeper);
+        list.remove(ZooDefs.CONFIG_NODE);
+        assertEquals(3, list.size());
+        assertIterableEquals(Arrays.asList("/", "/a", "/a/b"), list);
+    }
+
+    @Test
+    public void testListNoneRootPathSuccess() throws IOException, InterruptedException, KeeperException {
+        TestableZooKeeper zk = createClient();
+        zk.setData("/", "some".getBytes(), -1);
+        zk.create("/a", "some".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        zk.create("/a/b", "some".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
+        List<String> aList = ZKUtil.listSubTreeBFS(zk, "/a");
+        assertEquals(2, aList.size());
+        assertIterableEquals(Arrays.asList("/a", "/a/b"), aList);
+
+        List<String> bList = ZKUtil.listSubTreeBFS(zk, "/a/b");
+        assertEquals(1, bList.size());
+        assertIterableEquals(Collections.singletonList("/a/b"), bList);
+    }
 
     @Test
     public void testDeleteRecursiveInAsyncMode() throws Exception {
@@ -104,7 +136,7 @@ public class ZKUtilTest extends ClientBase {
 
     // batchSize>0 is async mode otherwise it is sync mode
     private void testDeleteRecursiveInSyncAsyncMode(int batchSize)
-      throws IOException, InterruptedException, KeeperException {
+        throws IOException, InterruptedException, KeeperException {
         TestableZooKeeper zk = createClient();
         String parentPath = "/a";
         zk.create(parentPath, "".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
@@ -112,7 +144,7 @@ public class ZKUtilTest extends ClientBase {
         List<Op> ops = new ArrayList<>();
         for (int i = 0; i < numberOfNodes; i++) {
             ops.add(Op.create(parentPath + "/a" + i, "".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE,
-              CreateMode.PERSISTENT));
+                CreateMode.PERSISTENT));
         }
         zk.multi(ops);
         ops.clear();
@@ -125,7 +157,7 @@ public class ZKUtilTest extends ClientBase {
         String subNode = "/a/a0";
         for (int i = 0; i < numberOfNodes; i++) {
             ops.add(Op.create(subNode + "/b" + i, "".getBytes(), ZooDefs.Ids.OPEN_ACL_UNSAFE,
-              CreateMode.PERSISTENT));
+                CreateMode.PERSISTENT));
         }
         zk.multi(ops);
 
